@@ -1,12 +1,12 @@
 ;;; abs-mode.el --- Major mode for the modeling language Abs -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2010-2022  Rudolf Schlatte
+;; Copyright (C) 2010-2022, 2026  Rudolf Schlatte
 
 ;; Author: Rudi Schlatte <rudi@constantly.at>
 ;; URL: https://github.com/abstools/abs-mode
-;; Package-Version: 1.7
-;; Package-Revision: v1.7-0-g0132032d9317
-;; Package-Requires: ((emacs "26.1") (erlang "2.8") (maude-mode "0.3") (flymake "1.0") (yasnippet "0.14.0"))
+;; Package-Version: 20260211.1320
+;; Package-Revision: aaf97a11ef79
+;; Package-Requires: ((emacs "27.1") (erlang "2.8") (maude-mode "0.3") (flymake "1.0") (yasnippet "0.14.0"))
 ;; Keywords: languages
 
 ;; This program is free software; you can redistribute it and/or modify
@@ -48,15 +48,16 @@
   "Major mode for editing files in the programming / modeling language Abs."
   :group 'languages)
 
-(defcustom abs-target-language 'erlang
-  "The default target language for code generation."
-  :type '(radio (const maude)
-                (const java)
-                (const erlang)
-                (const prolog))
+(defconst abs--backends '(java erlang maude prolog)
+  "List of ABS backends.")
+
+(define-obsolete-variable-alias 'abs-target-language 'abs-backend "v1.8")
+(defcustom abs-backend 'erlang
+  "The default backend for code generation."
+  :type `(radio ,@(mapcar (lambda (backend) (list 'const backend)) abs--backends))
   :group 'abs)
-(put 'abs-target-language 'safe-local-variable
-     #'(lambda (x) (member x '(maude java erlang prolog))))
+(put 'abs-backend 'safe-local-variable
+     #'(lambda (x) (member x abs--backends)))
 
 (defcustom abs-compiler-program "absc"
   "Command to invoke the Abs compiler.
@@ -118,8 +119,8 @@ well.  The Maude backend will use a default value of 100 in case
 (put 'abs-clock-limit 'safe-local-variable '(lambda (x) (or (integerp x) (null x))))
 
 (defcustom abs-local-port nil
-  "Port where to start the REST API / visualization server (Erlang backend).
-Server will not be started if nil."
+  "Port where to start the Model API.
+If nil, do not start the Model API."
   :type 'integer
   :group 'abs)
 (put 'abs-local-port 'safe-local-variable 'integerp)
@@ -131,16 +132,18 @@ Server will not be started if nil."
 (put 'abs-compile-with-coverage-info 'safe-local-variable 'booleanp)
 
 (defcustom abs-default-resourcecost 0
-  "Default resource cost of executing one ABS statement in the timed interpreter."
+  "Default resource cost of executing one ABS statement in the timed interpreter.
+Only effective for the Maude backend."
   :type 'integer
   :group 'abs)
 (put 'abs-default-resourcecost 'safe-local-variable 'integerp)
 
 (defcustom abs-link-source-path nil
   "Path to link the ABS runtime sources for the erlang backend.
-This enables development of the erlang backend by symlinking its
-sources into the generated code.  Sources will not be linked if
-NIL."
+This is a debugging aid for the erlang backend, symlinking the
+erlang runtime library sources into the generated code so that
+the model does not need to be recompiled when editing the Erlang
+runtime library.  If nil, sources will not be linked."
   :type '(choice (const :tag "Do not link" nil)
                  directory)
   :group 'abs)
@@ -155,7 +158,9 @@ Location of absfrontend.jar when installed via
 (put 'abs-directory 'risky-local-variable t)
 
 (defvar abs-product-name nil
-  "Product to be generated when compiling.")
+  "The product to compile from the software product line.
+When nil, ignore deltas and product declarations and compile the
+base model.")
 (put 'abs-product-name 'safe-local-variable 'stringp)
 
 ;;; Making faces
@@ -240,7 +245,7 @@ Location of absfrontend.jar when installed via
    (list (concat "\\(" abs--id-regexp "\\)[[:space:]]*(") 1
          'abs-function-name-face)
    (cons (concat "\\(" abs--id-regexp "\\)") 'abs-variable-name-face)
-   (list "\\<\\(# \w+\\)\\>" 1 'font-lock-warning-face t))
+   (list "\\<\\(# \\w+\\)\\>" 1 'font-lock-warning-face t))
   "Abs keywords.")
 
 ;;; cc-mode wants different fontification levels, but we only offer one.
@@ -337,6 +342,15 @@ Location of absfrontend.jar when installed via
      1))
   "Imenu expression for `abs-mode'.  See `imenu-generic-expression'.")
 
+;;; Minimal outline (folding) support
+(defvar abs--outline-regexp
+  (rx bol (or "delta" "def" "data" "type" "exception" "class" "interface" "module")))
+(defvar abs--outline-level (lambda () (1+ (/ (current-indentation) abs-indent))))
+
+(defun abs--read-backend ()
+  "Prompt the user to choose a backend."
+  (let ((backend-name (completing-read "Backend: " abs--backends nil t nil nil abs-backend)))
+    (intern-soft backend-name)))
 ;;; Minimal auto-insert mode support
 (define-auto-insert 'abs-mode '("Module name: " "module " str ";" ?\n ?\n))
 
@@ -471,6 +485,15 @@ buffer:
 (put 'abs-input-files 'safe-local-variable
      (lambda (list) (cl-every #'stringp list)))
 
+(defvar abs-modelapi-index-file nil
+  "The Model API index file to compile into this model, if any.")
+(put 'abs-modelapi-index-file 'safe-local-variable #'stringp)
+
+(defvar abs-modelapi-static-dir nil
+  "The Model API static directory containing to compile into this model, if any.")
+(put 'abs-modelapi-static-dir 'safe-local-variable #'stringp)
+
+
 (defvar abs-compile-command nil
   "The compile command called by \\[abs-next-action].
 The default behavior is to call \"make\" if a Makefile is in the
@@ -548,7 +571,7 @@ Try to return the correct value both in case
 decide."
   (file-name-as-directory
    (or abs-output-directory
-       (pcase abs-target-language
+       (pcase abs-backend
          (`maude ".")
          ;; to be adjusted if the ABS compiler changes its default output dirs.
          (`erlang "gen/erl/")
@@ -566,48 +589,51 @@ decide."
 
 (defun abs--calculate-compile-command ()
   "Return the command to compile the current model.
-Expects `abs-target-language' to be bound to the desired
-backend."
+Expects `abs-backend' to be bound to the desired backend."
   (cond (abs-compile-command)
         ((file-exists-p "Makefile") compile-command)
         (t (concat abs-compiler-program
-                   " --" (symbol-name abs-target-language)
+                   " --" (symbol-name abs-backend)
                    " "
                    ;; FIXME: make it work with filenames with spaces
                    (mapconcat (lambda (s) (concat "\"" s "\""))
                               (abs--input-files) " ")
-                   (when (eql abs-target-language 'maude)
+                   (when abs-modelapi-index-file
+                     (concat " --modelapi-index-file \"" abs-modelapi-index-file "\""))
+                   (when abs-modelapi-static-dir
+                     (concat " --modelapi-static-dir \"" abs-modelapi-static-dir "\""))
+                   (when (eql abs-backend 'maude)
                      (concat " -o \"" (abs--maude-filename) "\""))
-                   (when (and abs-java-output-jar-file (eql abs-target-language 'java))
+                   (when (and abs-java-output-jar-file (eql abs-backend 'java))
                      (concat " -o \"" abs-java-output-jar-file "\""))
                    (when abs-output-directory
                      (concat " -d \"" abs-output-directory "\""))
                    (when abs-product-name
                      (concat " --product " abs-product-name))
-                   (when (and (eql abs-target-language 'maude)
+                   (when (and (eql abs-backend 'maude)
                               (or abs-use-timed-interpreter
                                   (local-variable-p 'abs-clock-limit)))
                      (concat " --timed --limit="
                              (number-to-string (or abs-clock-limit 100))))
-                   (when (and (eql abs-target-language 'maude)
+                   (when (and (eql abs-backend 'maude)
                               (< 0 abs-default-resourcecost))
                      (concat " --defaultcost "
                              (number-to-string abs-default-resourcecost)))
-                   (when (and (eq abs-target-language 'erlang) abs-compile-with-coverage-info)
+                   (when (and (eq abs-backend 'erlang) abs-compile-with-coverage-info)
                      " --debuginfo")
                    ;; this branch must be last since it invokes a second
                    ;; command after `absc'
-                   (when (and (eq abs-target-language 'erlang)
+                   (when (and (eq abs-backend 'erlang)
                               abs-link-source-path)
                      (concat " && cd \"" (abs--real-output-directory) "\" && ./link_sources " abs-link-source-path))
                    " "))))
 
 (defun abs--needs-compilation ()
   "Return non-nil if current file needs to be (re)compiled.
-Expects `abs-target-language' to be bound to the desired backend."
+Expects `abs-backend' to be bound to the desired backend."
   (let* ((abs-output-file
           (abs--absolutify-filename
-           (pcase abs-target-language
+           (pcase abs-backend
              (`maude (abs--maude-filename))
              (`erlang (concat (abs--real-output-directory) "absmodel/Emakefile"))
              (`java (concat (abs--real-output-directory) "ABS/StdLib/Bool.java"))
@@ -625,14 +651,13 @@ Expects `abs-target-language' to be bound to the desired backend."
   "Compile the current buffer after prompting for a compile command.
 The user will be shown a hopefull-correct compile command in the
 minibuffer and can edit it before compilation starts.  Expects
-`abs-target-language' to be bound to the desired backend."
+`abs-backend' to be bound to the desired backend."
   (let ((compile-command (abs--calculate-compile-command)))
     (call-interactively 'compile)))
 
 (defun abs--compile-model-no-prompt ()
   "Compile the current buffer.
-Expects `abs-target-language' to be bound to the desired
-backend."
+Expects `abs-backend' to be bound to the desired backend."
   (compile (abs--calculate-compile-command)))
 
 ;;; Pacify the byte compiler.
@@ -641,9 +666,8 @@ backend."
 
 (defun abs--run-model ()
   "Start the model.
-Expects `abs-target-language' to be bound to the desired
-backend."
-  (pcase abs-target-language
+Expects `abs-backend' to be bound to the desired backend."
+  (pcase abs-backend
     (`maude (save-excursion (run-maude))
             (comint-send-string inferior-maude-buffer
                                 (concat "in \""
@@ -682,22 +706,25 @@ backend."
                   (buffer-name (concat "*abs java " module "*"))
                   (command (concat "java -cp gen:"
                                    (expand-file-name abs-java-classpath)
-                                   " " module ".Main &")))
+                                   " " module ".Main"
+                                   (when abs-clock-limit (format " -l %d " abs-clock-limit))
+                                   (when abs-local-port (format " -p %d " abs-local-port))
+                                   " &")))
              (when (get-buffer buffer-name)
                (kill-buffer (get-buffer buffer-name)))
              (let ((buffer (get-buffer-create buffer-name)))
                (pop-to-buffer buffer)
                (shell-command command buffer))))
-    (_ (error "Don't know how to run with target %s" abs-target-language))))
+    (_ (error "Don't know how to run with target %s" abs-backend))))
 
 (defun abs-next-action (flag)
   "Compile or execute the buffer.
 
 The language backend for compilation can be chosen by giving a
 `C-u' prefix to this command.  The default backend is set via
-customizing or setting `abs-target-language' and can be
-overridden for a specific abs file by giving a file-local value
-via `add-file-local-variable'.
+customizing or setting `abs-backend' and can be overridden for a
+specific abs file by giving a file-local value via
+`add-file-local-variable'.
 
 To compile or run a model that consists of more than one file,
 set `abs-input-files' to a list of filenames.
@@ -714,12 +741,8 @@ Erlang Emacs mode are installed.
 Argument FLAG will prompt for language backend to use if 1, i.e.,
 if the command was invoked with `C-u'."
   (interactive "p")
-  (let ((abs-target-language
-         (if (= 1 flag)
-             abs-target-language
-           (intern (completing-read "Target language: "
-                                    '("erlang" "java" "maude")
-                                    nil t nil nil "erlang")))))
+  (let ((abs-backend
+         (if (= 1 flag) abs-backend (or (abs--read-backend) abs-backend))))
     (if (abs--needs-compilation)
         (abs--compile-model)
       (abs--run-model))))
@@ -775,23 +798,23 @@ A definition can be interface, class, datatype or function."
 (easy-menu-define abs-mode-menu abs-mode-map
   "Abs mode menu."
   '("Abs"
-    ["Compile" (abs--compile-model-no-prompt abs-target-language) :active t]
-    ["Run" (abs--run-model abs-target-language)
-     :active (not (abs--needs-compilation abs-target-language))]
+    ["Compile" (abs--compile-model-no-prompt abs-backend) :active t]
+    ["Run" (abs--run-model abs-backend)
+     :active (not (abs--needs-compilation abs-backend))]
     "---"
     ("Select Backend"
-     ["Maude" (setq abs-target-language 'maude)
+     ["Maude" (setq abs-backend 'maude)
       :active t
       :style radio
-      :selected (eq abs-target-language 'maude)]
-     ["Erlang" (setq abs-target-language 'erlang)
+      :selected (eq abs-backend 'maude)]
+     ["Erlang" (setq abs-backend 'erlang)
       :active t
       :style radio
-      :selected (eq abs-target-language 'erlang)]
-     ["Java" (setq abs-target-language 'java)
+      :selected (eq abs-backend 'erlang)]
+     ["Java" (setq abs-backend 'java)
       :active t
       :style radio
-      :selected (eq abs-target-language 'java)])
+      :selected (eq abs-backend 'java)])
     ("Maude Backend Options"
      ["Timed interpreter"
       (setq abs-use-timed-interpreter (not abs-use-timed-interpreter))
@@ -834,6 +857,10 @@ The following keys are set:
   ;; imenu
   (setq imenu-generic-expression abs-imenu-generic-expression)
   (setq imenu-syntax-alist abs-imenu-syntax-alist)
+  ;; outline (code folding)
+  (setq-local outline-regexp abs--outline-regexp)
+  (setq-local outline-level abs--outline-level)
+  (outline-minor-mode)
   ;; speedbar support
   (when (fboundp 'speedbar-add-supported-extension)
     (speedbar-add-supported-extension ".abs"))
